 'PTP2 Paint Demo (C) Copyright 2011 Rayslogic.com, LLC
'Version 2.0
' A simple paint app demo for the PTP2 showing graphics and touchscreen interface
' Rev2 now uses version of Manta's assembly I2C driver for faster startup and easier touchscreen access

CON

  _clkmode = xtal1+pll16x
  _clkfreq = 80_000_000


  'LCD is 320x240 giving 20x15 tiles
  cols = 20
  rows = 15
  screensize = cols * rows

CON  'Constants for Graphics Demo
  x_tiles = 15
  y_tiles = 12
  GraphicsBytes=x_tiles*y_tiles*64
  _stack = ( GraphicsBytes + 100) >> 2   'accomodate display memory and stack  

  paramcount = 14       
  bitmap_base = $8000-GraphicsBytes    'these are the same because not double buffering...
  display_base = $8000-GraphicsBytes

  lines = 5
  thickness = 2

  GraphicsXOffset=1
  GraphicsYOffset=2
 

VAR
  'Variables for display driver 
  long  lcd_status      'status: off/visible/invisible  read-only       (5 contiguous longs)
  long  lcd_enable      'enable: off/on                 write-only
  long  lcd_videobase   'video base @word               write-only
  long  lcd_colorbase   'color base @long               write-only
  long  lcd_backlight   'Used to control LCD backlight
  long  lcd_hc         'horizontal cells                        write-only
  long  lcd_vc         'vertical cells                          write-only
  long  lcd_hx         'horizontal cell expansion               write-only    
  long  lcd_vx         'vertical cell expansion                 write-only
  long  lcd_ho         'horizontal offset                       write-only
  long  lcd_vo         'vertical offset                         write-only

  word  screen[screensize]
  long  col, row, color
  long  boxcolor,ptr

  byte  x[lines]
  byte  y[lines]
  byte  xs[lines]
  byte  ys[lines]

var 'vars for Graphics Demo
  long  mousex, mousey
  long touchx, touchy
  'long touchx0,touchdx,touchy0,touchdy   'cal factors
  long CurrentColor, bLineMode, bStillDown, LastX, LastY

VAR
      'for custom characters
  word user_charbase

OBJ
  lcd : "PTP2_LcdDriver2a"  'LCD driver
  gr    : "graphics"
  mouse : "mouse"
  i2c: "PTP2_i2cDriver2a"  

pub Main|i
  'Prepare user characters (on screen buttons) for display (need to move so that last six bits of address are all zeroes!)
  '64 byte align the user characters 
  i:=(@UserCharEnd-@UserCharStart)/64  '#of new characters
  user_charbase := @UserCharStart & $FFC0 'destination
  longmove(user_charbase,@UserCharStart,16*i)

  i2c.initialize 'start up i2c driver and also initialize LCD (uses I2C i/o expander to init display's SPI interface) 
  
  'Start up Display
  'clear screen
  print($100)
  'start driver
  lcd_enable:=1
  lcd_videobase := @screen
  lcd_colorbase := @vgacolors
  lcd_backlight:=16  '1=bright, 16=medium, 32=dark
  lcd_hc:=cols
  lcd_vc:=rows
  lcd_hx:=lcd_vx:=1
  lcd_ho:=lcd_vo:=0
  lcd.start(@lcd_status)


  'Set Colors
  'set text color#0 to yellow with dark blue background
  lcd.SetTextColor(lcd#Yellow,lcd#DarkBlue,0,@vgacolors)          'blue background with yellow letters

  'Set Graphics Colors
  '#16=drawing color:  white,red,green,black
  SetGraphicsColor(16,lcd#White,lcd#Red,lcd#Green,lcd#Black)
  '#21=fill screen icon: white,blue,purple,blue
  SetGraphicsColor(21,lcd#White,lcd#DarkBlue,lcd#DarkCyan,lcd#DarkBlue)
  '#22=clear screen icon: black,blue,purple,blue
  SetGraphicsColor(22,lcd#Black,lcd#DarkBlue,lcd#DarkCyan,lcd#DarkBlue)
  'Other graphics colors are set in "Paint" routine 


  Paint
                             


PUB SetGraphicsColor(n,c3,c2,c1,c0) 'Sets graphics color, n, given 4 RGB colors
  VgaColors[n]:=c3<<24+c2<<16+c1<<8+c0  


PUB Paint|dx,dy,GraphicsColor,xc,yc,tx,ty

  print($100)
  col:=4
  PrintStr(string("PTP2 Paint!"))

  'init colors
  GraphicsColor:=16 'Using this color for graphics colors
  'vgacolors[GraphicsColor]:=$0CFCCC00  'white, blue, magenta, black
  
  'init tile screen
  repeat dx from 0 to x_tiles - 1
    repeat dy from 0 to y_tiles - 1
      screen[(dy+GraphicsYOffset) * cols + dx+GraphicsXOffset] := display_base >> 6 + dy + dx * y_tiles +GraphicsColor<<10


  'start and setup graphics
  gr.start
  gr.setup(x_tiles, y_tiles, x_tiles*8, y_tiles*8, bitmap_base)

  gr.color(2)
  gr.width(3)

    'Center of graphics area:  xc= GraphicsXOffset*16+x_tiles*16/2
    '                          yc= GraphicsYOffset*16+y_tiles*16/2
  xc:=GraphicsXOffset*16+x_tiles*16/2
  yc:=GraphicsYOffset*16+y_tiles*16/2

  SetColor(1)
  SetLineMode(true) 'select line or dot mode
  bStillDown:=false

  
  'Draw Color Buttons
  Bitmap2Bit(@ColorButton, 16, 4, 2, 2, 17)
  Bitmap2Bit(@ColorButton, 18, 4, 2, 2, 18)
  Bitmap2Bit(@ColorButton, 16, 6, 2, 2, 19)
  Bitmap2Bit(@ColorButton, 18, 6, 2, 2, 20)
  'Draw Fill/Clear Screen buttons 
  Bitmap2Bit(@ColorButton, 16, 8, 2, 2, 21)  'fill
  Bitmap2Bit(@ColorButton, 18, 8, 2, 2, 22)  'clear
  'Draw Line/Dot Mode Buttons
  Bitmap2Bit(@LineButton, 16, 10, 2, 2, 23)  'fill
  Bitmap2Bit(@DotsButton, 18, 10, 2, 2, 24)  'clear  

  WaitStylusUp  'Don't start to draw until stylus is up   
  repeat
    WaitStylusDown
    touchx:=i2c.GetTouchX
    touchy:=i2c.GetTouchY

    'See if any buttons were pushed
    tx:=touchx/16
    if tx>(GraphicsXOffset+x_tiles-1)
      'A button may have been pushed
      ty:=touchy/16
      CheckButtons(tx,ty)
    else
      'draw point    

       
      dx:=touchx-xc
      dy:=yc-touchy
      if bLineMode==true
        if bStillDown==true
            gr.line(dx,dy)
        else
          gr.plot(dx,dy)
      else
        gr.plot(dx,dy)
      LastX:=dx
      LastY:=dy
      bStillDown:=true 

PUB SetLineMode(bMode) 'select line or dot mode  
  if bMode==true
    bLineMode:=true
  ' '#23=line mode icon: white,blue,purple,blue
    SetGraphicsColor(23,lcd#white,lcd#DarkBlue,lcd#DarkCyan,lcd#DarkBlue)
    '#24=dots mode icon: white,blue,black,blue
    SetGraphicsColor(24,lcd#white,lcd#DarkBlue,lcd#Black,lcd#DarkBlue)

  else
    bLineMode:=false
    '#23=line mode icon: white,blue,black,blue
    SetGraphicsColor(23,lcd#white,lcd#DarkBlue,lcd#Black,lcd#DarkBlue)
    '#24=dots mode icon: white,blue,purple,blue
    SetGraphicsColor(24,lcd#white,lcd#DarkBlue,lcd#DarkCyan,lcd#DarkBlue)

     
PUB SetColor(c) 'set current color and adjust buttons
  CurrentColor:=c
  gr.color(3-c)
  
  if c==0
    '#17=white color button: white,white,purple,blue
    SetGraphicsColor(17,lcd#white,lcd#white,lcd#DarkCyan,lcd#DarkBlue)
  else
    '#17=white color button: white,white,black,blue
    SetGraphicsColor(17,lcd#white,lcd#white,lcd#Black,lcd#DarkBlue)

  if c==1
    '#18=red color button: red,red,purple,blue
    SetGraphicsColor(18,lcd#Red,lcd#Red,lcd#DarkCyan,lcd#DarkBlue)
  else
    '#18=red color button: red,red,black,blue
    SetGraphicsColor(18,lcd#Red,lcd#Red,lcd#Black,lcd#DarkBlue)
 
  if c==2
    '#19=green color button: green,green,purple,blue
    SetGraphicsColor(19,lcd#Green,lcd#Green,lcd#DarkCyan,lcd#DarkBlue)
  else
    '#19=green color button: green,green,black,blue
    SetGraphicsColor(19,lcd#Green,lcd#Green,lcd#Black,lcd#DarkBlue) 
    
  if c==3
    '#20=black color button: black,black,purple,blue
    SetGraphicsColor(20,lcd#Black,lcd#Black,lcd#DarkCyan,lcd#DarkBlue)
  else
    '#20=black color button: black,black,black,blue
    SetGraphicsColor(20,lcd#Black,lcd#Black,lcd#Black,lcd#DarkBlue) 

                            
PUB CheckButtons(xTile,yTile)|column,i
  'see if stylus pushed on a button
  if xTile>15 and xTile<20
    'May have been a button
    if xTile<18
      'first column
      if yTile==4 or yTile==5  'White
        SetColor(0)
      if yTile==6 or yTile==7 'Green
        SetColor(2)
      if yTile==8 or yTile==9 'Fill screen
        i:=3-CurrentColor
        i:=i<<6+i<<4+i<<2+i
        bytefill(bitmap_base,i,GraphicsBytes)
      if yTile==10 or yTile==11 'Line Mode
        SetLineMode(true) 
    if xTile>17
      'Second column
      if yTile==4 or yTile==5  'Red
        SetColor(1)
      if yTile==6 or yTile==7 'Black
        SetColor(3)
      if yTile==8 or yTile==9 'Clear screen     
        gr.clear
      if yTile==10 or yTile==11 'Dot Mode
        SetLineMode(false)     



PUB WaitStylusUp
  'Wait for stylus to be up
  repeat while i2c.PollTouchN(5)<>0

    
PUB WaitStylusDown|tz
  'Wait for stylus to be down and update bStillDown status
  repeat
    tz:=i2c.PollTouchN(10)
    if tz==0
      bStillDown:=false
    else
      quit

PRI Bitmap1Bit(pBitmap, xPos, yPos, xSize, ySize, clr)|c,i,j,BmpAddress
  row:=yPos
  col:=xPos
  c:=0
  BmpAddress:=pBitmap+user_charbase-@UserCharStart
  repeat j from 0 to (ySize-1) step 2
    repeat i from 0 to (xSize-1)
      screen[row * cols + col] := (clr<<1+1) << 10 + BmpAddress>>6 +c
      screen[(row+1) * cols + col] := (clr<<1) << 10 + BmpAddress>>6 +c
      c++
      col++
    row+=2
    col:=xPos

PRI Bitmap2Bit(pBitmap, xPos, yPos, xSize, ySize, clr)|c,i,j,BmpAddress
  row:=yPos
  col:=xPos
  c:=0
  BmpAddress:=pBitmap+user_charbase-@UserCharStart
  repeat j from 0 to (ySize-1)
    repeat i from 0 to (xSize-1)
      screen[row * cols + col] := (clr) << 10 + BmpAddress>>6 +c
      'Print2Bit(c,clr,pBitmap)
      c++
      col++
    row++
    col:=xPos

PRI PrintStr(stringptr)

'' Print a zero-terminated string

  repeat strsize(stringptr)
    print(byte[stringptr++])

PUB print(c) | i, k

  case c
    $00..$FF:           'character?
      k := color << 1 + c & 1
      i := k << 10 + $200 + c & $FE
      screen[row * cols + col] := i
      screen[(row + 1) * cols + col] := i | 1
      if ++col == cols
        newline

    $100:               'clear screen?
      wordfill(@screen, $220, screensize)
      col := row := 0

    $108:               'backspace?
      if col
        col--

    $10D:               'return?
      newline

    $110..$11F:         'select color?
      color := c & $F


' New line

PRI newline : i

  col := 0
  if (row += 2) == rows
    row -= 2
    'scroll lines
    repeat i from 0 to rows-3
'      wordmove(@screen[i*cols], @screen[(i+2)*cols], cols)
    'clear new line
'    wordfill(@screen[(rows-2)*cols], $200, cols<<1)


' Data

PRI dec(value) | i

'' Print a decimal number

  if value < 0
    -value
    print("-")

  i := 1_000_000_000

  repeat 10
    if value => i
      print(value / i + "0")
      value //= i
      result~~
    elseif result or i == 1
      print("0")
    i /= 10

PRI hex(value, digits)

'' Print a hexadecimal number

  value <<= (8 - digits) << 2
  repeat digits
    print(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))
    
DAT 'Default Touchscreen Calibrations
touchx0 long 2510
touchdx long 186
touchy0 long 2751
touchdy long 247

DAT  'Bitmap data
padding LONG  7[16] 'alignment padding for the following user defined characters
UserCharStart long
CalibrateCursor long 
file "calibrateCursor.dat"  '1 tile
ColorButton long
file "button1.dat"  '2x2=4 tiles
LineButton long
file "button2.dat"  '2x2=4 tiles
DotsButton long
file "button3.dat"  '2x2=4 tiles
UserCharEnd long

DAT   'colors

vgacolors               long  0[64]  'colors are defined programmatically